@@ -340,15 +340,15 @@ g_unix_input_stream_read (GInputStream  *stream,
 
   unix_stream = G_UNIX_INPUT_STREAM (stream);
 
-  if (cancellable)
+  if (g_cancellable_make_pollfd (cancellable, &poll_fds[1]))
     {
       poll_fds[0].fd = unix_stream->priv->fd;
       poll_fds[0].events = G_IO_IN;
-      g_cancellable_make_pollfd (cancellable, &poll_fds[1]);
       do
 	poll_ret = g_poll (poll_fds, 2, -1);
       while (poll_ret == -1 && errno == EINTR);
-      
+      g_cancellable_release_fd (cancellable);
+
       if (poll_ret == -1)
 	{
           int errsv = errno;
